"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        lock_transforms.py
Date:           11/03/14
Description:    This script will lock/unlock the selected nodes.
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import lock_transforms
                    lock_transforms.main()
"""
import maya.cmds as cmds


class LockTransforms(object):
    def __init__(self):
        self.mode_radio = None
        self.translate_checkbox = None
        self.rotate_checkbox = None
        self.scale_checkbox = None
        self.attributes_checkbox = None
        self.selection_only_checkbox = None
        self.lock_button = None
        self.status_label = None

    @staticmethod
    def set_transform_locks(object_list, transform_list, lock_state):
        """Locks/Unlocks transform properties"""
        for current in object_list:
            for current_transform in transform_list:
                if cmds.attributeQuery(current_transform, node=current, exists=True):
                    cmds.setAttr(current + '.' + current_transform, lock=lock_state)

    @staticmethod
    def set_attribute_locks(object_list, lock_state):
        """Locks/Unlocks node attributes including transforms"""
        for current in object_list:
            for current_attr in cmds.listAttr(current, settable=True, leaf=True):
                try:
                    cmds.setAttr(current + '.' + current_attr, lock=lock_state)
                except:
                    pass
            # cmds.lockNode(current, lock=lock_state, ignoreComponents=True)

    def get_transform_names(self):
        """Returns a list of transform names that must be locked/unlocked"""
        names = []

        if cmds.checkBox(self.translate_checkbox, query=True, value=True):
            names.extend(['translateX', 'translateY', 'translateZ'])
        if cmds.checkBox(self.rotate_checkbox, query=True, value=True):
            names.extend(['rotateX', 'rotateY', 'rotateZ'])
        if cmds.checkBox(self.scale_checkbox, query=True, value=True):
            names.extend(['scaleX', 'scaleY', 'scaleZ'])

        return names

    def set_locks(self):
        """Sets the locks for all objects"""
        lock_state = cmds.radioButton(self.mode_radio, query=True, select=True)

        if cmds.checkBox(self.selection_only_checkbox, query=True, value=True):
            lock_objects = cmds.ls(selection=True, long=True, noIntermediate=True)
        else:
            lock_objects = cmds.ls(dag=True, long=True, noIntermediate=True)

        cmds.text(self.status_label, edit=True, label='INFO: Processing objects. Please wait...')

        if cmds.checkBox(self.attributes_checkbox, query=True, value=True):
            self.set_attribute_locks(lock_objects, lock_state)
        else:
            self.set_transform_locks(lock_objects, self.get_transform_names(),
                                     lock_state)

        if lock_state:
            message = 'INFO: ' + str(len(lock_objects)) + ' objects locked'
        else:
            message = 'INFO: ' + str(len(lock_objects)) + ' objects unlocked'
        cmds.text(self.status_label, edit=True, label=message)

    def setup_ui(self):
        """Creates the UI"""
        if cmds.window('lockTransformsWindow', exists=True):
            cmds.deleteUI('lockTransformsWindow')
            
        lock_window = cmds.window('lockTransformsWindow', title='Lock Transforms',
                                  width=195, height=275, minimizeButton=False,
                                  maximizeButton=False, sizeable=False,
                                  resizeToFitChildren=True)
        cmds.columnLayout(width=190)
        cmds.frameLayout(width=190, label='Mode',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.rowColumnLayout(width=180, numberOfColumns=2,
                             columnWidth=[(1, 60), (2, 60)])
        cmds.radioCollection()
        self.mode_radio = cmds.radioButton(label='Lock', select=True,
                                           changeCommand=self.set_button_label)
        cmds.radioButton(label='Unlock')
        cmds.setParent('..')  # end rowColumnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.frameLayout(width=190, label='Transforms',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=180, rowSpacing=5)
        self.translate_checkbox = cmds.checkBox(label='Translate', value=True)
        self.rotate_checkbox = cmds.checkBox(label='Rotate', value=True)
        self.scale_checkbox = cmds.checkBox(label='Scale', value=True)
        self.attributes_checkbox = cmds.checkBox(label='All Transforms and Attributes',
                                                 changeCommand=self.set_checkbox_state)
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.frameLayout(width=190, label='Options', collapsable=False,
                         marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=180)
        self.selection_only_checkbox = cmds.checkBox(label='Selection Only', value=True)
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.columnLayout(width=190, rowSpacing=5, columnAttach=['left', 5])
        cmds.text(height=5, label='')
        self.lock_button = cmds.button(width=180, label='Lock Objects Now',
                                       command=self.validate_user_settings)
        self.status_label = cmds.text(width=180, label='INFO: Standing by...',
                                      align='left')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end columnLayout
        cmds.showWindow(lock_window)

    def set_button_label(self, *args):
        """Mode changed event handler. Changes the lock button text"""
        if cmds.radioButton(self.mode_radio, query=True, select=True):
            cmds.button(self.lock_button, edit=True, label='Lock Objects Now')
        else:
            cmds.button(self.lock_button, edit=True,
                        label='Unlock Objects Now')

    def set_checkbox_state(self, *args):
        """Attribute checkbox event handler. Enables/Disables the transform
        checkboxes"""
        enable_state = cmds.checkBox(self.attributes_checkbox, query=True, value=True)
        cmds.checkBox(self.translate_checkbox, edit=True, enable=(not enable_state))
        cmds.checkBox(self.rotate_checkbox, edit=True, enable=(not enable_state))
        cmds.checkBox(self.scale_checkbox, edit=True, enable=(not enable_state))

    def validate_user_settings(self, *args):
        """Validates the user settings before locking/unlocking nodes"""
        is_valid = False

        if cmds.checkBox(self.translate_checkbox, query=True, value=True):
            is_valid = True
        if cmds.checkBox(self.rotate_checkbox, query=True, value=True):
            is_valid = True
        if cmds.checkBox(self.scale_checkbox, query=True, value=True):
            is_valid = True
        if cmds.checkBox(self.attributes_checkbox, query=True, value=True):
            is_valid = True

        if is_valid:
            self.set_locks()
        else:
            cmds.text(self.status_label, edit=True, label='ERROR: No transforms checked')


def main():
    """Main entry point. Opens the UI"""
    lt = LockTransforms()
    lt.setup_ui()


if __name__ == '__main__':
    main()
